import os
import copy
import shutil
import json
import pkg_resources
import subprocess
import shlex
import xarray as xr
from pathlib import Path
from typing import List

from .plotting import Plotting
from .input_configure import TimeSeries

PARAMETER = pkg_resources.resource_filename(
        __name__, 'meta/parameter_meta.json')
with open(PARAMETER, 'r') as f:
    PARAMETER_META = json.load(f)
FILE = pkg_resources.resource_filename(
        __name__, 'meta/file_name.json')
with open(FILE, 'r') as f:
    FILE_NAME = json.load(f)

class Simulation():
    """The simulation object provides a wrapper for RHESSys simulations"""

    def __init__(self, executable, project_path, run_suffix='rhessys_run', initialize=True, config_dir='.pyrhessys'):
        """Initialize a new simulation object"""
        self.executable = executable
        self.path = project_path
        self.clim = self.path + '/clim'
        self.defs = self.path + '/defs'
        self.flows = self.path + '/flows'
        self.obs = self.path + '/obs'
        self.output = self.path + '/output'
        self.tecfiles = self.path + '/tecfiles'
        self.worldfiles = self.path + '/worldfiles'
        self.parameters = PARAMETER_META
        self.file_name = FILE_NAME
        self.project_path = Path(os.path.abspath(os.path.realpath(project_path)))
        self.config_path = self.project_path / config_dir
        self.plotting = Plotting(self.obs)
        self.input_ts = TimeSeries(project_path)
        self.status = 'Uninitialized'
        self.stdout = None
        self.stderr = None
        self.process = None
        if initialize:
            self.initialize()

    def initialize(self):
        self.status = 'Initialized'

    def apply_config(self, config):
        for k, v in config.get('parameters', {}).items():
            self.parameters[k] = v
     
    def _gen_rhessys_cmd(self, run_suffix, processes=1, progress='m'):
        self.run_suffix = run_suffix
        rhessys_run_cmd = ''.join(['cd {}; ./{} -st {} -ed {}'.format(os.path.dirname(self.executable), self.parameters['version'], self.parameters['start_date'], self.parameters['end_date']),
                           ' -b -gwtoriparian -t {}{}'.format(self.tecfiles, self.file_name['tecfiles']),
                           ' -w {}{} -whdr {}{}'.format(self.worldfiles, self.file_name['world'], self.worldfiles, self.file_name['world_hdr']),
                           ' -r {}{} {}{}'.format(self.flows, self.file_name['flows_sub'], self.flows, self.file_name['flows_surf']),
                           ' -pre {}/{} -gw {} {} -s {} {} {}'.format(self.output, run_suffix, self.parameters['gw1'], self.parameters['gw2'], self.parameters['s1'], self.parameters['s2'], self.parameters['s3']),     
                           ' -snowEs {} -snowTs {} -sv {} {} -svalt {} {}'.format(self.parameters['snowEs'], self.parameters['snowTs'], self.parameters['sv1'], self.parameters['sv2'], self.parameters['svalt1'], self.parameters['svalt2'])                          
                          ])

        if self.parameters['locationid'] == "0" :
            patch_cmd = ""
        elif self.parameters['locationid'] == "1" :
            patch_cmd = " -p"
        elif self.parameters['locationid'] == "2" :
            patch_cmd = " -p {} {} {} {}".format(self.parameters['basin_id'], self.parameters['hillslope_id'], self.parameters['zone_id'], self.parameters['patch_id'])
        else:
            print(" set locationid: 0-No use location ID, 1-Use every location ID, 2-Use certain ID")
        print(rhessys_run_cmd + patch_cmd)
        return rhessys_run_cmd + patch_cmd

    def _run_local(self, run_suffix, processes=1, progress=None):
        """Start a local simulation"""
        run_cmd = self._gen_rhessys_cmd(run_suffix, processes, progress)
        self.process = subprocess.Popen(run_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        #print(self.process.communicate())
        #output = self.process.communicate()[0].decode('utf-8')
        #print(output)
        self.status = 'Running'

    def start(self, run_option,  run_suffix='rhessys_run', processes=1, progress=None):
        """Run a RHESSys simulation"""
        self.run_suffix = run_suffix
        if run_option == 'local':
            self._run_local(run_suffix, processes, progress)
        elif run_option == 'docker':
            self._run_docker(run_suffix, processes, progress)
        else:
            raise NotImplementedError('Invalid runtime given! '
                                      'Valid options: local, docker')

    def run(self, run_option,  run_suffix='rhessys_run', processes=1, progress=None):
        self.start(run_option, run_suffix, processes, progress)
        self.monitor()

    def monitor(self):
        # Simulation already run
        if self.status in ['Error', 'Success']:
            return self.status

        if self.process is None:
            raise RuntimeError('No simulation started! Use simulation.start '
                               'or simulation.execute to begin a simulation!')

        self.stdout, self.stderr = self.process.communicate()
        if isinstance(self.stdout, bytes):
            self.stderr = self.stderr.decode('utf-8', 'ignore')
            self.stdout = self.stdout.decode('utf-8', 'ignore')

        SUCCESS_MSG = 'Constructing basins'
        print(self.stdout)
        if SUCCESS_MSG not in self.stdout:
            self.status = 'Error'
        else:
            self.status = 'Success'

        try:
            self._output = [xr.open_dataset(f) for f in self.get_output()]
            if len(self._output) == 1:
                self._output = self._output[0]
        except Exception:
            self._output = None

        return self.status

    def get_output(self) -> List[str]:
        new_file_text = 'Created output file:'
        out_files = []
        for l in self.stdout.split('\n'):
            if new_file_text in l:
                out_files.append(
                    l.split(';')[0].replace(new_file_text, '').strip())
        return out_files

    def parallel_job(self, executable, start_date, end_date, config, path):

        list_keys = [k for k in config]
        p = list_keys[0].split("++")[1:len(list_keys[0].split("++")) - 1]

        parameter = []
        name = []

        for i in list_keys:
            name.append(i)

        for i, value in enumerate(list_keys):
            parameter.append(list_keys[i].split("++")[1:len(list_keys[i].split("++")) - 1])

        parallel_run_cmd1 = []
        for i, k in enumerate(list_keys):
            rhessys_run_cmd = ''.join(['cd {}; ./{} -st {} -ed {}'.format(os.path.dirname(self.executable), self.parameters['version'], self.parameters['start_date'], self.parameters['end_date']),
                           ' -b -gwtoriparian -t {}{}'.format(self.tecfiles, self.file_name['tecfiles']),
                           ' -w {}{} -whdr {}{}'.format(self.worldfiles, self.file_name['world'], self.worldfiles, self.file_name['world_hdr']),
                           ' -r {}{} {}{}'.format(self.flows, self.file_name['flows_sub'], self.flows, self.file_name['flows_surf']),
                           ' -pre {}/{}'.format(self.output, name[i])  
                          ])
            parallel_run_cmd1.append(rhessys_run_cmd)
        parallel_run_cmd2 = []
        my_dict = {'gw1': self.parameters['gw1'], 'gw2': self.parameters['gw2'], 's1': self.parameters['s1'], 's2': self.parameters['s2'], 
                's3': self.parameters['s3'], 'snowEs': self.parameters['snowEs'], 'snowTs': self.parameters['snowTs'], 
                'sv1': self.parameters['sv1'], 'sv2': self.parameters['sv2'], 'svalt1': self.parameters['svalt1'], 'svalt2': self.parameters['svalt2']}

        for i, k in enumerate(parameter):
            for j in parameter[i]:
                keys = j.split("=")[0]
                values = j.split("=")[1]
                my_dict[keys] = values

            par = ' -gw ' + str(my_dict['gw1']) + ' ' + str(my_dict['gw2']) + ' ' + ' -s ' + str(my_dict['s1']) + ' ' + str(my_dict['s2']) + ' ' + str(my_dict['s3'])  \
                + ' ' +' -snowEs ' + str(my_dict['snowEs']) + ' ' +' -snowTs '+ str(my_dict['snowTs']) + ' ' + ' -sv ' \
                + str(my_dict['sv1']) + ' ' + str(my_dict['sv2']) + ' -svalt ' + str(my_dict['svalt1']) + ' ' + str(my_dict['svalt2'])
                                                                                  
            parallel_run_cmd2.append(par)  

            if self.parameters['locationid'] == "0" :
                patch_cmd = ""
            elif self.parameters['locationid'] == "1" :
                patch_cmd = " -p"
            elif self.parameters['locationid'] == "2" :
                patch_cmd = " -p {} {} {} {}".format(self.parameters['basin_id'], self.parameters['hillslope_id'], self.parameters['zone_id'], self.parameters['patch_id'])
            else:
                print(" set locationid: 0-No use location ID, 1-Use every location ID, 2-Use certain ID")

        for i, k in enumerate(list_keys):
            cmd = parallel_run_cmd1[i] + ' ' + parallel_run_cmd2[i] + ' ' + patch_cmd 
            print(cmd)

            file_name = path + "/" + "input_"+str(i)+".txt"
            parallel_cmd = open(file_name, "w") 
            parallel_cmd.write(cmd)
            parallel_cmd.close()
        
        return name


import os
import shutil
import pkg_resources

lulc_forest_fraction = pkg_resources.resource_filename(
        __name__, 'meta/lulc_forest_fraction.csv')
lulc_urban_fraction = pkg_resources.resource_filename(
        __name__, 'meta/lulc_urban_fraction.csv')
lulc_difficult_run_fraction = pkg_resources.resource_filename(
        __name__, 'meta/lulc_difficult_run_fraction.csv')
lulc_meadow_creek_fraction = pkg_resources.resource_filename(
        __name__, 'meta/lulc_meadow_creek_fraction.csv')
vegCollection = pkg_resources.resource_filename(
        __name__, 'meta/vegCollection.csv')
soilCollection = pkg_resources.resource_filename(
        __name__, 'meta/soilCollection.csv')
lulcCollectionEC = pkg_resources.resource_filename(
        __name__, 'meta/lulcCollectionEC.csv')
zoneCollection = pkg_resources.resource_filename(
        __name__, 'meta/zoneCollection.csv')
hillCollection = pkg_resources.resource_filename(
        __name__, 'meta/hillCollection.csv')


import os
import pandas as pd
import numpy as np
import shutil
import stat
import inspect
import subprocess
from functools import partial
from pathlib import Path
from pkg_resources import resource_filename as resource
from pyrhessys import Simulation
from string import Template
from typing import List, Dict

def read_template(path):
    with open(path, 'r') as f:
        OST_FILE= f.read()
    return Template(OST_FILE)

resource = partial(resource, __name__)

# Paths to template files
INPT_FILE = resource('template/ostIn.template')
EXEC_FILE = resource('template/model_executable.template')
SAVE_FILE = resource('template/save_parameters.template')

# Templates
INPT_META = read_template(INPT_FILE)
EXEC_META = read_template(EXEC_FILE)
SAVE_META = read_template(SAVE_FILE)

PARAMETER = resource('meta/parameter_meta.json')


class Ostrich():
    """
    Provides a high level interface to the OSTRICH optimization package.
    This class can currently only be used for single-objective optimization
    using the DDS algorithm as defined in the template file. Currently the
    metrics calculated are KGE, MAE, and MSE as defined in the evaluation
    package, though more metrics can be implemmented quite easily.
    A basic workflow for this object is:
        ::
        import pyrhessys as pr
        rhessys_exe = './rhessys.exe'
        ostrich_exe = './ostrich.exe'
        project_path = 'project_directory'
        python_exe = '/pool0/data/andrbenn/.conda/all/bin/python'
        ostrich = ps.Ostrich(ostrich_exe, rhessys_exe, project_path, python_path=python_exe)
        ostrich.calib_params = [
            ps.OstrichParam('paramName', starValue, (minValue, maxValue)),
        ]
        ostrich.obs_data_file = 'obs_data.nc'
        ostrich.sim_calib_var = 'sim_varname'
        ostrich.obs_calib_var = 'obs_varname'
        ostrich.write_config()
        ostrich.run()
    Attributes
    ----------
    ostrich:
        Path to OSTRICH executable
    python_path:
        Path to Python executable used for the ``run_script``
    summa:
        Path to the RHESSys executable
    template:
        OSTRICH configuration file template
    save_template:
        Template for script to save best parameters
    run_template:
        Template for script to run and evaluate RHESSys
    config_path:
        Path to location of calibration runs/logs
    simulation:
        pyrhessys Simulation object used as template
    file_manager:
        File manager file for RHESSys simulation
    seed:
        Random seed for calibration
    errval:
        Error value for OSTRICH
    perturb_val:
        Strength of parameter perturbations during calibration
    max_iters:
        Number of calibration trial runs
    cost_function:
        Metric to use when ranking calibration runs
    maximize:
        Whether to maximize the ``cost_function``
    simulation_kwargs:
        Keyword arguments to pass to the simulation run function
    """

    def __init__(self, ostrich_executable, rhessys_executable, project_path, python_path='python'):
        """Initialize a new Ostrich object"""
        self.available_metrics: np.ndarray = np.array(['KGE', 'MAE', 'MSE', 'RMSE', 'NSE'])
        self.ostrich: str = ostrich_executable
        self.python_path: str = python_path
        self.rhessys: str = rhessys_executable
        self.template: Template = INPT_META
        self.save_template: Template = SAVE_META
        self.run_template: Template = EXEC_META
        self.config_path: Path = Path(os.path.abspath(project_path)) / 'calibration'
        self.simulation = Simulation(rhessys_executable, project_path,
                                     config_dir=self.config_path)
        self.path = project_path
        self.run_script: Path = self.config_path / 'run_script.py'
        self.save_script: Path = self.config_path / 'save_script.py'
        self.metrics_file: Path = self.config_path / 'metrics.txt'
        self.metrics_log: Path = self.config_path / 'metrics_log.csv'
        self.impot_strings: str = ''
        self.conversion_function: callable = lambda x: x
        self.filter_function: callable = lambda x, y: (x, y)
        self.preserve_output: str ='no'
        self.seed: int = 42
        self.errval: float = -9999
        self.perturb_val: float = 0.2
        self.max_iters: int = 100
        self.calib_params: List[OstrichParam] = []
        self.tied_params: List[OstrichTiedParam] = []
        self.cost_function: str = 'KGE'
        self.objective_function: str = 'gcop'
        self.maximize: bool = True
        self.simulation_kwargs: Dict = {}
        self.allow_failures: bool = False

    def run(self, prerun_cmds=[], monitor=True):
        """Start calibration run"""
        if len(prerun_cmds):
            preprocess_cmd = " && ".join(prerun_cmds) + " && "
        else:
            preprocess_cmd = ""
        cmd = preprocess_cmd + f'cd {str(self.config_path)} && ./ostrich'
        self.cmd = cmd
        self.process = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE, shell=True)
        if monitor:
            self.stdout, self.stderr = self.process.communicate()
            if isinstance(self.stdout, bytes):
                self.stderr = self.stderr.decode('utf-8', 'ignore')
                self.stdout = self.stdout.decode('utf-8', 'ignore')

    def monitor(self):
        if not self.process:
            return
        else:
            self.stdout, self.stderr = self.process.communicate()
            if isinstance(self.stdout, bytes):
                self.stderr = self.stderr.decode('utf-8', 'ignore')
                self.stdout = self.stdout.decode('utf-8', 'ignore')
        return self.stdout, self.stderr

    def write_config(self):
        """Writes all necessary files for calibration"""
        if not os.path.exists(self.config_path):
            os.mkdir(self.config_path)

        # Substitue templates and save
        self.weightTemplateFile = self.write_weight_template_section()
        self.weightValueFile = self.write_weight_value_section()

        with open(self.config_path / 'ostIn.txt', 'w') as f:
            f.write(self.template.substitute(self.map_vars_to_template))
        with open(self.save_script, 'w') as f:
            f.write(self.save_template.substitute(self.map_vars_to_save_template))

        #self.simulation._write_configuration()
        with open(self.run_script, 'w') as f:
            f.write(self.run_template.substitute(self.map_vars_to_run_template))

        shutil.copy(self.ostrich, self.config_path / 'ostrich')

        # Make sure we set permissions for execution
        st = os.stat(self.config_path / 'ostrich')
        os.chmod(self.config_path / 'ostrich', st.st_mode | stat.S_IEXEC)
        st = os.stat(self.run_script)
        os.chmod(self.run_script, st.st_mode | stat.S_IEXEC)
        st = os.stat(self.save_script)
        os.chmod(self.save_script, st.st_mode | stat.S_IEXEC)

    def write_weight_template_section(self, file_name=Path('param_mapping.tpl')) -> Path:
        """Write the parameter name mapping for OSTRICH"""
        with open(self.config_path / file_name, 'w') as f:
            for cp in self.calib_params:
                if cp.weightname.endswith('mtp'):
                    f.write(f'{cp.realname} | {cp.weightname}\n')
            for tp in self.tied_params:
                if tp.realname.endswith('mtp'):
                    f.write(f'{tp.realname.replace("_mtp", "")} | {tp.realname}\n')
        return Path('.') / file_name

    def write_weight_value_section(self, file_name='param_weights.txt') -> Path:
        """Write the parameter values for OSTRICH"""
        with open(self.config_path / file_name, 'w') as f:
            f.write('\n'.join([f'{cp.realname} | {cp.value}'
                               for cp in self.calib_params]) + '\n')
        return Path('.') / file_name

    def add_tied_param(self, param_name, lower_bound, upper_bound, initial_value=0.5):
        self.calib_params.append(OstrichParam(f'{param_name}', initial_value, (0.01, 0.99), weightname=f'{param_name}_scale'))
        self.tied_params.append(OstrichTiedParam(param_name, lower_bound, upper_bound))

    @property
    def param_section(self) -> str:
        """Write list of calibration parameters"""
        return '\n'.join(str(param) for param in self.calib_params)

    @property
    def tied_param_section(self) -> str:
        """Write list of tied calibration parameters"""
        if len(self.tied_params):
            return '\n'.join(str(param) for param in self.tied_params)
        else:
            return '# nothing to do here'

    @property
    def response_section(self) -> str:
        """Write section of OSTRICH configuration for selecting metric"""
        metric_row = np.argwhere(self.cost_function == self.available_metrics)[0][0]
        return f"{self.cost_function} {self.metrics_file}; OST_NULL {metric_row} 1 ' '"

    @property
    def tied_response_section(self) -> str:
        """Write section for determining if we are maximizing or minimizing the metric"""
        if self.maximize:
            return f'neg{self.cost_function} 1 {self.cost_function} wsum -1.00'
        else:
            return '# nothing to do here'

    @property
    def map_vars_to_template(self):
        """For completion of the OSTRICH input template"""
        return {'runScript': self.run_script,
                'objectiveFun': self.objective_function,
                'saveScript': self.save_script,
                'preserveOutput': self.preserve_output,
                'seed': self.seed,
                'errval': self.errval,
                'perturbVal': self.perturb_val,
                'maxIters': self.max_iters,
                'paramSection': self.param_section,
                'tiedParamSection': self.tied_param_section,
                'responseSection': self.response_section,
                'tiedResponseSection': self.tied_response_section,
                'costFunction': f'neg{self.cost_function}' if self.maximize else self.cost_function,
                'weightTemplateFile': self.weightTemplateFile,
                'weightValueFile': self.weightValueFile
                }

    @property
    def map_vars_to_save_template(self):
        """For completion of the parameter saving template"""
        return {
                'pythonPath': self.python_path,
                'saveDir': self.config_path.parent / 'best_calibration',
                'modelDir': self.config_path}

    @property
    def map_vars_to_run_template(self):
        """For completion of the model run script template"""
        return {
                'pythonPath': self.python_path,
                'rhessysExe': self.rhessys,
                'path': self.path,
                'obsDataFile': self.obs_data_file,
                'simVarName': self.sim_calib_var,
                'obsVarName': self.obs_calib_var,
                'outFile': self.metrics_file,
                'metricsLog': self.metrics_log,
                'importStrings': self.import_strings,
                'conversionFunc': "=".join(inspect.getsource(self.conversion_function).split('=')[1:]),
                'filterFunc': "=".join(inspect.getsource(self.filter_function).split('=')[1:]),
                'paramMappingFile': self.weightTemplateFile,
                'paramWeightFile': self.weightValueFile,
                'simulationArgs': self.simulation_kwargs,
                'allowFailures': self.allow_failures,
                'paramFile': PARAMETER,
                }

class OstrichParam():
    """
    Definition of a RHESSys parameter to be optimized by OSTRICH
    Parameters
    ----------
    realname:
        Parameter name as seen by RHESSys
    weightname:
        Parameter name as seen by OSTRICH
    value:
        Default value
    lower:
        Lower bound for parameter value
    upper:
        Upper bound for parameter value
    """

    def __init__(self, name, value, val_range, weightname=''):
        self.realname = name
        if not weightname:
            self.weightname = f'{name}_mtp'
        else:
            self.weightname = weightname
        self.value = value
        self.lower, self.upper = val_range

    def __str__(self):
        return f"{self.weightname} {self.value} {self.lower} {self.upper} none none none free"


class OstrichTiedParam():
    def __init__(self, name, lower_param, upper_param):
        self.realname = f'{name}_mtp'
        self.weightname = f'{name}_scale'
        self.lower_param = f'{lower_param}_mtp'
        self.upper_param = f'{upper_param}_mtp'

    @property
    def type_data(self):
        """This corresponds to the equation y = x2 + x1x3 - x1x2"""
        if self.lower_param and self.upper_param:
            return "ratio 0 -1 1 0 0 1 0 0 0 0 0 0 0 0 0 1 free"
        elif self.lower_param:
            raise NotImplementedError()
            return ""
        elif self.upper_param:
            raise NotImplementedError()
            return ""

    def __str__(self):
        return f"{self.realname} 3 {self.weightname} {self.lower_param} {self.upper_param} {self.type_data}"


import xarray as xr
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import json
import pkg_resources
import seaborn as sns
import cartopy
import geoviews as gv
import geopandas as gpd
import holoviews as hv
import pandas as pd
import xarray as xr

OUTPUT = pkg_resources.resource_filename(
        __name__, 'meta/output_variables.json')
with open(OUTPUT, 'r') as f:
    OUTPUT_VARIABLE = json.load(f)

class Plotting:

	def __init__(self, filepath):
		self.filepath = filepath
		#self.start_date = PARAMETER_META['start_date']
		#self.end_date = PARAMETER_META['end_date']

	def ts_plot(self, data, sim_output_variable, sim_label, pre_trim: int=0, post_trim: int=-1):
		self.filepath
		# set output variables and variable description
		y_axis = OUTPUT_VARIABLE[sim_output_variable]['description']+'('+OUTPUT_VARIABLE[sim_output_variable]['Units']+')'
		# Plotting 
		plt.figure(figsize=(17,10))
		ax = plt.gca()
		ax.plot(data["Date"][pre_trim:post_trim], data[sim_output_variable][pre_trim:post_trim], label=sim_label)
		ax.grid(True)
		ax.set_ylabel(y_axis, fontsize=18)
		plt.xticks(fontsize=18)
		plt.yticks(fontsize=18)
		ax.legend(loc='upper left', bbox_to_anchor=(1, 1), fontsize=10)
		plt.savefig("figure.png")

	def ts_plot_obs(self, sim_data, sim_output_variable, sim_label, obs_data, obs_variable: str="", obs_label: str="", pre_trim: int=0, post_trim: int=-1):
		self.filepath
		# set output variables and variable description
		y_axis = OUTPUT_VARIABLE[sim_output_variable]['description']+'('+OUTPUT_VARIABLE[sim_output_variable]['Units']+')'
		# Plotting 
		plt.figure(figsize=(17,10))
		ax = plt.gca()
		ax.plot(sim_data["Date"][pre_trim:post_trim], sim_data[sim_output_variable][pre_trim:post_trim], label=sim_label)
		ax.plot(obs_data.index[pre_trim:post_trim], obs_data[obs_variable][pre_trim:post_trim], label=obs_label)
		ax.grid(True)
		ax.set_ylabel(y_axis, fontsize=18)
		plt.xticks(fontsize=18)
		plt.yticks(fontsize=18)
		ax.legend(loc='upper left', bbox_to_anchor=(1, 1), fontsize=10)
		plt.savefig("figure.png")

	def ensemble_plot(self, sim_data, name_list, num_config, sim_output_variable, pre_trim: int=0, post_trim: int=-1):
		LINE_STYLES = ['solid', 'dashed', 'dashdot', 'dashdot', 'dotted']
		NUM_STYLES = len(LINE_STYLES)

		sns.reset_orig()  # get default matplotlib styles back
		clrs = sns.color_palette("bright", n_colors=num_config)  # a list of RGB tuples
		fig, ax = plt.subplots(1,figsize=(15,7),linewidth=3.0)
		for i in range(num_config):
			lines = ax.plot(sim_data[i]["Date"][pre_trim:post_trim].values, sim_data[i][sim_output_variable][pre_trim:post_trim].values)
			lines[0].set_color(clrs[i])
			lines[0].set_linestyle(LINE_STYLES[i%NUM_STYLES])
		y_axis = OUTPUT_VARIABLE[sim_output_variable]['description']+'('+OUTPUT_VARIABLE[sim_output_variable]['Units']+')'
		# add x, y label
		ax.set_xlabel('Date (day)', fontsize=15)
		ax.set_ylabel(y_axis, fontsize=15)
		# show up the legend
		ax.tick_params(labelsize=15)
		ax.grid('on')
		fig.legend(labels=name_list, bbox_to_anchor=(0.82, 0.82))
		plt.show()
		plt.savefig("figure.png")

	def ensemble_obs_plot(self, sim_data, name_list, num_config, sim_output_variable, obs_data, obs_variable, pre_trim: int=0, post_trim: int=-1):
		LINE_STYLES = ['solid', 'dashed', 'dashdot', 'dashdot', 'dotted']
		NUM_STYLES = len(LINE_STYLES)

		sns.reset_orig()  # get default matplotlib styles back
		clrs = sns.color_palette("bright", n_colors=num_config)  # a list of RGB tuples
		fig, ax = plt.subplots(1,figsize=(15,7),linewidth=3.0)
		for i in range(num_config):
			lines = ax.plot(sim_data[i]["Date"][pre_trim:post_trim].values, sim_data[i][sim_output_variable][pre_trim:post_trim].values)
			lines[0].set_color(clrs[i])
			lines[0].set_linestyle(LINE_STYLES[i%NUM_STYLES])
		y_axis = OUTPUT_VARIABLE[sim_output_variable]['description']+'('+OUTPUT_VARIABLE[sim_output_variable]['Units']+')'
		ax.scatter(sim_data[i]["Date"][pre_trim:post_trim].values, obs_data[obs_variable][pre_trim:post_trim-1].values, color='blue', s=10)
		# add x, y label
		ax.set_xlabel('Date (day)', fontsize=15)
		ax.set_ylabel(y_axis, fontsize=15)
		# show up the legend
		ax.tick_params(labelsize=15)
		ax.grid('on')
		fig.legend(labels=name_list, bbox_to_anchor=(0.82, 0.82))
		plt.show()
		plt.savefig("figure.png")
	#def daily_patch_map(netcdf, variable, timestep):
    	# Convert the data from an xarray dataset to a pandas dataframe 
    	# This is necessary for geoviews to be able to plot it
	#	out_df = netcdf[variable].sel(time=timestep).to_dataframe()
		# Make sure we have some metadata to join with the shapefile
	#	out_df['gridcode'] = shapes_df['gridcode'].values
    	# Create the shape plot - some keys:
    	#  - shapes.records() provides the geometry from the shapefiles
    	#  - out_df provides the data
    	#  - value=var chooses which column of the dataframe to use as data for coloring
    	#  - index=['gridcode'] will provide a name on mouse hover
   		#  - on='gridcode' is the key to join the `shapes` with `out_df`
	#	poly = gv.Shape.from_records(shapes.records(), out_df, value=variable, index=['gridcode'], on='gridcode')
    	# Add some options to make things a bit nicer
    	#  - width=700, sets the width, as we expect
    	#  - cmap='plasma' sets the colormap to 'plasma'
    	#  - tools=['hover'] provides information on mouse hover over
    	#  - colorbar=True adds a colorbar
    	#  - alpha=0.7 adds a bit of transparency
	#	poly = poly.opts(width=700, height=600, cmap='plasma', tools=['hover'], colorbar=True, alpha=0.7)
	#	return poly	

	#def daily_patch_map_timeseries(netcdf, variable, start, stop, step, patch):
		# Just as before, use the `var_map` function to build our map
	#	poly = hv.HoloMap({t: daily_var_map(variable, t) for t in netcdf.time.values[start:stop:step]}, kdims=['timestep'])
    	# Vlines will give us an indicator of which time slice we are looking at
	#	vlines = hv.HoloMap({t: hv.VLine(t) for t in netcdf.time.values[start:stop:step]}, kdims=['timestep'])
    	# This seems to make the plot appear more often - probably a holo/geoviews bug somewhere 
	#	poly
    
    	# Calculate min, max, and mean over the domain for each time
	#	vmin = netcdf[variable].isel(time=slice(start, stop)).min(dim=patch).rolling(time=step).min().values
	#	vmax = netcdf[variable].isel(time=slice(start, stop)).max(dim=patch).rolling(time=step).max().values
	#	vmean = netcdf[variable].isel(time=slice(start, stop)).mean(patch).rolling(center=True, time=step).mean().dropna('time')
   
    	# Build the complete interactive plot. Layers are as follows
    	#  - gv.tile_sources.EsriTerrain- Add a background with topography
    	#  - * poly - overlay our map onto the background
    	#  - + hv.Area... - add the area plot to the right
    	#  - * hv.Curve... - overlay the mean curve onto the area plot
    	#  - * vlines... - overlay the timestep indicator
	#	return (gv.tile_sources.EsriTerrain
    #        	* poly
    #        	+ (hv
    #          		.Area((cw18.time[start:stop], vmin, vmax), vdims=['vmin', 'vmax'])
    #           		.opts(alpha=0.5, color='gold', line_color=None)
    #           		.redim.label(x='Date', vmin=var.capitalize()))
    #        	* hv.Curve(vmean).opts(color='purple', alpha=0.8)
    #        	* vlines.opts(alpha=0.4, color='red'))


import pandas as pd
import numpy as np
import os, shutil


class TimeSeries:

	def __init__(self, filepath):
		self.filepath = filepath

	def rainfall(self, name, data, start_date, col_name="rain"):
		np.savetxt(name+'.rain', data[col_name].values, fmt='%2.4f', header=start_date, comments='')
		if os.path.exists(self.filepath + '/' + name+'.rain'):
			os.remove(self.filepath + '/' + name+'.rain')
		shutil.move(name+'.rain', self.filepath)

	def tempmax(self, name, data, start_date, col_name="tmax"):
		np.savetxt(name+'.tmax', data[col_name].values, fmt='%2.1f', header=start_date, comments='')
		if os.path.exists(self.filepath + '/' + name+'.tmax'):
			os.remove(self.filepath + '/' + name+'.tmax')
		shutil.move(name+'.tmax', self.filepath)

	def tempmin(self, name, data, start_date, col_name="tmin"):
		np.savetxt(name+'.tmin', data[col_name].values, fmt='%2.1f', header=start_date, comments='')
		if os.path.exists(self.filepath + '/' + name+'.tmin'):
			os.remove(self.filepath + '/' + name+'.tmin')
		shutil.move(name+'.tmin', self.filepath)

	def vpd(self, name, data, start_date, col_name="vpd"):
		#vpd = pd.to_numeric(data[col_name].values, errors='coerce')
		#np.savetxt(r'cwt.vpd', vpd, fmt='%3.2f', header=start_date, comments='')
		np.savetxt(name+'.vpd', data[col_name].values, fmt='%2.1f', header=start_date, comments='')
		if os.path.exists(self.filepath + '/' + name+'.vpd'):
			os.remove(self.filepath + '/' + name+'.vpd')
		shutil.move(name+'.vpd', self.filepath)

	def rh(self, name, data, start_date, col_name="rh"):
		#rh = pd.to_numeric(data[col_name].values, errors='coerce')
		#np.savetxt(r'cwt.relative_humidity', rh, fmt='%2.1f', header=start_date, comments='')
		np.savetxt(name+'.relative_humidity', data[col_name].values, fmt='%2.1f', header=start_date, comments='')
		if os.path.exists(self.filepath + '/' + name+'.relative_humidity'):
			os.remove(self.filepath + '/' + name+'.relative_humidity')
		shutil.move(name+'.relative_humidity', self.filepath)

	def kdownDirect(self, name, data, start_date, col_name="kdownDirect"):
		#kdownDirect = pd.to_numeric(data[col_name].values, errors='coerce')
		#np.savetxt(r'cwt.Kdown_direct', kdownDirect, fmt='%5.1f', header=start_date, comments='')
		np.savetxt(name+'.Kdown_direct', data[col_name].values, fmt='%2.1f', header=start_date, comments='')
		if os.path.exists(self.filepath + '/' + name+'.Kdown_direct'):
			os.remove(self.filepath + '/' + name+'.Kdown_direct')
		shutil.move(name+'.Kdown_direct', self.filepath)

	def clim_base(self, name, sta_id="101", x="278391.71", y="3882439.5",z="638.0", e_lai="2.0", s_height="22.9"):
		base = open(name+".base", "w")
		contents = ''.join(['{} base_station_id \n'.format(sta_id),
                            '{} x_coordinate \n'.format(x),
                            '{} y_coordinate \n'.format(y),
                            '{} z_coordinate \n'.format(z),
                            '{} effective_lai \n'.format(e_lai),
                            '{} screen_height \n'.format(s_height),
                            '{}/{} daily_climate_prefix \n'.format(self.filepath, name),
                            '0'])
		base.writelines(contents)
		base.close()
		if os.path.exists(self.filepath + '/' + name+'.base'):
			os.remove(self.filepath + '/' + name+'.base')
		shutil.move(name+".base", self.filepath)

	def tecfile(self, start_date):
		tec_daily = open("tec_daily.txt","w") 
		contents = ["{} print_daily_on ".format(start_date)]  
		tec_daily.writelines(contents) 
		tec_daily.close() 
		if os.path.exists(self.filepath + '/tec_daily.txt'):
			os.remove(self.filepath + '/tec_daily.txt')
		shutil.move('tec_daily.txt', self.filepath)


from copy import deepcopy
from distributed import Client, get_client
import os
import pandas as pd
import time
import xarray as xr
import json
import pkg_resources

from .simulation import Simulation
from .utils import ChainDict, product_dict

OMP_NUM_THREADS = int(os.environ.get('OMP_NUM_THREADS', 1))

PARAMETER = pkg_resources.resource_filename(
        __name__, 'meta/parameter_meta.json')
with open(PARAMETER, 'r') as f:
    PARAMETER_META = json.load(f)

class Ensemble(object):
    '''
    Ensembles represent an multiple RHESSys configurations based on
    changing the decisions or parameters of a given run.
    '''

    def __init__(self, executable: str, configuration: dict,
                 path: str=None, num_workers: int=1,
                 threads_per_worker: int=OMP_NUM_THREADS,
                 scheduler: str=None, client: Client=None):
        """
        Create a new Ensemble object. The API mirrors that of the
        Simulation object.
        """
        self._status = 'Initialized'
        self.executable: str = executable
        self.path: str = path
        self.configuration: dict = configuration
        self.num_workers: int = num_workers
        self.simulations: dict = {}
        self.submissions: list = []
        self.parameters = PARAMETER_META

        # Try to get a client, and if none exists then start a new one
        if client:
            self._client = client
            workers = len(self._client.get_worker_logs())
            if workers <= self.num_workers:
                self._client.cluster.scale(workers)
        else:
            try:
                self._client = get_client()
                # Start more workers if necessary:
                workers = len(self._client.get_worker_logs())
                if workers <= self.num_workers:
                    self._client.cluster.scale(workers)
            except ValueError:
                self._client = Client(n_workers=self.num_workers,
                                  threads_per_worker=threads_per_worker)
        self._generate_simulation_objects()

    def _generate_simulation_objects(self):
        """
        Create a mapping of configurations to the simulation objects.
        """
        self.name_list = []
        for name, config in self.configuration.items():
            self.name = name
            self.name_list.append(self.name)
            self.simulations[name] = Simulation(
                self.executable, self.path, run_suffix=self.name)

    def _generate_coords(self):
        """
        Generate the coordinates that can be used to merge the output
        of the ensemble runs into a single dataset.
        """
        parameter_dims = ChainDict()
        for name, conf in self.configuration.items():
            for k, v in conf.get('parameters', {}).items():
                parameter_dims[k] = v
        return {'parameters': parameter_dims}

    def merge_output(self):
        """
        Open and merge all of the output datasets from the ensemble
        run into a single dataset.
        """
        sim_output = []
        for lists in self.name_list:
            plot_data = pd.read_csv(self.path + '/output/'+ lists +'_basin.daily', delimiter=" ")
            date_index = pd.date_range(self.parameters['start_date'][0:10], self.parameters['end_date'][0:10], freq='1D')
            plot_data.insert(loc=0, column='Date', value=date_index[:-1].values)
            plot_data.set_index('Date')
            sim_output.append(plot_data)
        return sim_output

    def start(self, run_option: str, prerun_cmds: list=None):
        """
        Start running the ensemble members.

        Parameters
        ----------
        run_option:
            The run type. Should be either 'local' 
        prerun_cmds:
            A list of preprocessing commands to run
        """
        for n, s in self.simulations.items():
            # Sleep calls are to ensure writeout happens
            config = self.configuration[n]
            self.submissions.append(self._client.submit(
                _submit, s, n, run_option, config))

    def run(self, run_option: str, prerun_cmds=None, monitor: bool=True):
        """
        Run the ensemble

        Parameters
        ----------
        run_option:
            Where to run the simulation. Can be ``local`` 
        prerun_cmds:
            A list of shell commands to run before running RHESSys
        monitor:
            Whether to halt operation until runs are complete
        """
        self.start(run_option, prerun_cmds)
        if monitor:
            return self.monitor()
        else:
            return True

    def monitor(self):
        """
        Halt computation until submitted simulations are complete
        """
        simulations = self._client.gather(self.submissions)
        for s in simulations:
            self.simulations[s.run_suffix] = s

    def summary(self):
        """
        Show the user information about ensemble status
        """
        success, error, other = [], [], []
        for n, s in self.simulations.items():
            if s.status == 'Success':
                success.append(n)
            elif s.status == 'Error':
                error.append(n)
            else:
                other.append(n)
        return {'success': success, 'error': error, 'other': other}

    def rerun_failed(self, run_option: str, prerun_cmds=None, monitor: bool=True):
        """
        Try to re-run failed simulations.

        Parameters
        ----------
        run_option:
            Where to run the simulation. Can be ``local`` or ``docker``
        prerun_cmds:
            A list of shell commands to run before running RHESSys
        monitor:
            Whether to halt operation until runs are complete
        """
        run_summary = self.summary()
        self.submissions = []
        for n in run_summary['error']:
            config = self.configuration[n]
            s = self.simulations[n]
            s.reset()
            self.submissions.append(self._client.submit(
                _submit, s, n, run_option, prerun_cmds, config))
        if monitor:
            return self.monitor()
        else:
            return True


def _submit(s: Simulation, name: str, run_option: str, config):
    s.initialize()
    s.apply_config(config)
    s.run(run_option, run_suffix=name)
    s.process = None
    return s

def parameter_product(list_config):
    return {'++'+'++'.join('{}={}'.format(k, v) for k, v in d.items())+'++':
            {'parameters': d} for d in product_dict(**list_config)}



import os
import shutil
import pkg_resources

basin_extraction = pkg_resources.resource_filename(
        __name__, 'r_code/basin_extraction.R')
zone_cluster = pkg_resources.resource_filename(
        __name__, 'r_code/zone_cluster.R')
ssurgo_properties_extraction1 = pkg_resources.resource_filename(
        __name__, 'r_code/ssurgo_properties_extraction1.R')
ssurgo_properties_extraction2 = pkg_resources.resource_filename(
        __name__, 'r_code/ssurgo_properties_extraction2.R')
ssurgo_properties_extraction3 = pkg_resources.resource_filename(
        __name__, 'r_code/ssurgo_properties_extraction3.R')
patch_lulc_extraction = pkg_resources.resource_filename(
        __name__, 'r_code/patch_lulc_extraction.R')  
lulcFrac_write2gis = pkg_resources.resource_filename(
        __name__, 'r_code/lulcFrac_write2gis.R')   
g2world_def_flow = pkg_resources.resource_filename(
        __name__, 'r_code/g2world_def_flow.R')
g2world = pkg_resources.resource_filename(
        __name__, 'r_code/g2world.R')
elevation_analysis = pkg_resources.resource_filename(
        __name__, 'r_code/elevation_analysis.R')
road_storm_drainage_outlet = pkg_resources.resource_filename(
        __name__, 'r_code/road_storm_drainage_outlet.R')
LIB_misc = pkg_resources.resource_filename(
        __name__, 'r_code/LIB_misc.R')
ssurgo_extraction = pkg_resources.resource_filename(
        __name__, 'r_code/ssurgo_extraction.R')
ssurgo_soiltexture2gis = pkg_resources.resource_filename(
        __name__, 'r_code/ssurgo_soiltexture2gis.R')



import subprocess
import shlex
import itertools
import numpy as np
import time

def execute(executable, start_date, end_date, config, path, option="series"):

    default_cmd = '-b -newcaprise -capr 0.001 -gwtoriparian -capMax 0.01 -slowDrain -leafDarkRespScalar 0.5   -frootRespScalar 0.25 -StemWoodRespScalar 0.05'

    list_keys = [k for k in config]
    p = list_keys[0].split("++")[1:len(list_keys[0].split("++")) - 1]

    parameter = []
    name = []

    for i in list_keys:
        name.append(i)

    for i, value in enumerate(list_keys):
        parameter.append(list_keys[i].split("++")[1:len(list_keys[i].split("++")) - 1])


    ensemble_rhessys_run_cmd = []
    for i, k in enumerate(list_keys):
        rhessys_run_cmd = '{} -st {} -ed {} {} -t {}tecfiles/tec_daily.txt ' \
                             '-w {}worldfiles/worldfile -whdr {}worldfiles/worldfile.hdr ' \
                             '-r {}flows/flowtable.txt -rtz 2.7 -pre {}output/{}' \
            .format(executable, start_date, end_date, default_cmd,
                    path, path, path, path, path, name[i])
        ensemble_rhessys_run_cmd.append(rhessys_run_cmd)

    my_dict = {'s1': 2.9, 's2': 1.4, 's3': 20.0, 'sv1': 4.5, 'sv2': 55.6, 'gw1': 0.05, 'gw2': 0.1}
    ensemble_parameters = []
    for i, k in enumerate(parameter):
        for j in parameter[i]:
            keys = j.split("=")[0]
            values = j.split("=")[1]
            my_dict[keys] = values
        par = ' -s ' + str(my_dict['s1']) + ' ' + str(my_dict['s2']) + ' ' + str(my_dict['s3']) + ' -sv ' \
              + str(my_dict['sv1']) + ' ' + str(my_dict['sv2']) + ' -gw ' + str(my_dict['gw1']) + ' ' + str(
            my_dict['gw2'])
        ensemble_parameters.append(par)

    for i, k in enumerate(list_keys):
        cmd = ensemble_rhessys_run_cmd[i] + ' ' + ensemble_parameters[i]
        if option == "series":
            print(ensemble_rhessys_run_cmd[i] + ensemble_parameters[i])
            cmd_rhessys = shlex.split(ensemble_rhessys_run_cmd[i] + ensemble_parameters[i])
            process = subprocess.Popen(cmd_rhessys, stdout=subprocess.PIPE)
            output = process.communicate()[0].decode('utf-8')
            
        elif option == "parallel":
            file_name = path + "parallel_input/" + "input_"+str(i)+".txt"
            parallel_cmd = open(file_name, "w") 
            parallel_cmd.write(cmd)
            parallel_cmd.close() 
    return name

def product_dict(**kwargs):
    """
    Take a set of dictionary arguments and generate a new set of
    dictionaries that have all combinations of values for each key.
    """
    keys, vals = kwargs.keys(), kwargs.values()
    for instance in itertools.product(*vals):
        yield dict(zip(keys, instance))


def parameter_product(list_config):
    return {'++'+'++'.join('{}={}'.format(k, v) for k, v in d.items())+'++':
            {'parameters': d} for d in product_dict(**list_config)}

def safe_arange(start, stop, step):
    a = np.arange(start, stop, step)
    result =[]
    for i in a:
        par = round(i, 10)
        result = np.append(result, par)
    return result


from .simulation import Simulation
from .ensemble import Ensemble
from .plotting import Plotting
from .read_rcode import *
from .read_meta import *
from .read_template import *
from .read_shellscript import *
from .utils import *
from .input_configure import *
from .ostrich import Ostrich, OstrichParam

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions


import os
import shutil
import pkg_resources

delineation_d8 = pkg_resources.resource_filename(
        __name__, 'sh_code/grass_delineation_d8.sh')
delineation_dinf = pkg_resources.resource_filename(
        __name__, 'sh_code/grass_delineation_dinf.sh')
spatial_hierarchy = pkg_resources.resource_filename(
        __name__, 'sh_code/grass_spatial_hierarchy.sh')
lulc_fraction = pkg_resources.resource_filename(
        __name__, 'sh_code/lulc_fraction.sh')



import itertools
import os
import shutil
from urllib.request import urlretrieve
import subprocess
from hs_restclient import HydroShare
import pandas as pd
import xarray as xr
import numpy as np


def get_hs_resource(resource_id, file_path):
    hs = HydroShare()
    hs.getResource(resource_id, destination=file_path, unzip=True)

    # unpack the simulation archive and remove unncessary files
    hs_resource_dir = os.path.join(file_path, resource_id, resource_id, 'data/contents/')
    hs_resource = os.listdir(hs_resource_dir)
    shutil.unpack_archive(hs_resource_dir+hs_resource[0], extract_dir=file_path)
    cmd = "rm -rf " + os.path.join(file_path, resource_id)
    subprocess.run(cmd, shell=True)

def replace_string(file, current_string, new_string):
    if not os.path.isfile(file):
        print ("Error on replace_string, not a regular file: "+file)
        sys.exit(1)

    f1=open(file,'r').read()
    f2=open(file,'w')
    m=f1.replace(current_string,new_string)
    f2.write(m)

def complie(file_path, version_option="rhessys5.20.0.develop"):
    if version_option:
        complie_RHESSys = 'cd ' + file_path +'/RHESSysEastCoast;make'
        subprocess.run(complie_RHESSys, shell=True)
        exe_RHESSys = 'cd ' + file_path +'/RHESSysEastCoast;chmod +rwx ' + version_option
        subprocess.run(exe_RHESSys, shell=True)
        execution_file = file_path + '/RHESSysEastCoast/' + version_option

    else:
        print("You have to set the exact RHESSys version")

    return execution_file

def product_dict(**kwargs):
    """
    Take a set of dictionary arguments and generate a new set of
    dictionaries that have all combinations of values for each key.
    """
    keys, vals = kwargs.keys(), kwargs.values()
    for instance in itertools.product(*vals):
        yield dict(zip(keys, instance))

def netcdf_output(model_output, start_date, end_date, patch, shapes_df, output_name):
    # set the value of dimension and coordinate for NetCDF
    dates = pd.date_range(start_date, end_date)
    shape = (len(dates), len(shapes_df))
    dims = ('time', patch, )
    patchs = shapes_df['gridcode'].tolist()
    coords = {'time': dates, patch: patchs}     

    # Create the data structure of RHESSys mode output
    cw18 = xr.Dataset(coords=coords)
    # Check All RHESSys output variables except date variables
    all_var_col = model_output.columns.values
    var_col = all_var_col[4:]
    for varname in var_col:
        cw18[varname] = xr.DataArray(data=np.full(shape, np.nan),
                                     coords=coords, dims=dims,
                                     name=varname)
        # get values of variable and insert new multi-array model output considering patch and time dimension into xarray Dataset
        single_model_output = model_output[varname].values
        multi_array_model_output = single_model_output.reshape(cw18.dims['time'], cw18.dims[patch])
        cw18[varname].values[:, :] = multi_array_model_output
    # create new NetCDF file of RHESSys output at Coweeta subbasin 18
    cw18.to_netcdf(output_name)    


class ChainDict(dict):
    """
    A dictionary which instead of overwriting on existing keys,
    will instead store the values in a list.
    """
    def __setitem__(self, key, val):
        newval = [val]
        if key in list(self.keys()):
            oldval = self.__getitem__(key)
            newval = list(set(oldval + newval))
        dict.__setitem__(self, key, newval)



from sklearn.metrics import mean_absolute_error, mean_squared_error
import math
import numpy as np

def trim_time(sim, obs, roundto='min'):
    #sim['time'] = sim['time'].dt.round(roundto)
    #obs['time'] = obs['time'].dt.round(roundto)
    sim_start = str(sim.index[0])[0:10]    #str(sim_ds1.index[0])[0:10]
    sim_stop = str(sim.index[-1])[0:10]
    obs_start = str(obs.index[0])[0:10]
    obs_stop = str(obs.index[-1])[0:10]
    #start = max(sim_start, obs_start)
    #stop = min(sim_stop, obs_stop)
    return sim_start, sim_stop


def kling_gupta_efficiency(sim, obs):
    obs = obs.values
    sim = sim.values
    #obs = np.asarray(obs)
    #sim = np.asarray(sim)
    obs_filtered = obs[np.logical_and(~np.isnan(obs), ~np.isnan(sim))]
    sim_filtered = sim[np.logical_and(~np.isnan(obs), ~np.isnan(sim))]
    sim_std = np.std(sim_filtered, ddof=1)
    obs_std = np.std(obs_filtered, ddof=1)
    sim_mu = np.mean(sim_filtered)
    obs_mu = np.mean(obs_filtered)
    r = np.corrcoef(sim_filtered, obs_filtered)[0, 1]
    var = sim_std / obs_std
    bias = sim_mu / obs_mu
    kge = 1 - np.sqrt((bias-1)**2 + (var-1)**2 + (r-1)**2)
    return kge


def decomposed_kling_gupta_efficiency(sim, obs):
    obs = obs.values
    sim = sim.values
    #obs = np.asarray(obs)
    #sim = np.asarray(sim)
    obs_filtered = obs[np.logical_and(~np.isnan(obs), ~np.isnan(sim))]
    sim_filtered = sim[np.logical_and(~np.isnan(obs), ~np.isnan(sim))]
    sim_std = np.std(sim_filtered, ddof=1)
    obs_std = np.std(obs_filtered, ddof=1)
    sim_mu = np.mean(sim_filtered)
    obs_mu = np.mean(obs_filtered)
    r = np.corrcoef(sim_filtered, obs_filtered)[0, 1]
    var = sim_std / obs_std
    bias = sim_mu / obs_mu
    kge = 1 - np.sqrt((bias-1)**2 + (var-1)**2 + (r-1)**2)
    return kge, bias, var, r


def nash_sutcliffe_efficiency(sim, obs):
    obs = obs.values
    sim = sim.values
    #obs = np.asarray(obs)
    #sim = np.asarray(sim)
    obs_filtered = obs[np.logical_and(~np.isnan(obs), ~np.isnan(sim))]
    sim_filtered = sim[np.logical_and(~np.isnan(obs), ~np.isnan(sim))]
    obs_mu = np.mean(obs_filtered)
    num = np.sum( (sim_filtered - obs_filtered) ** 2 )
    den = np.sum( (obs_filtered - obs_mu) ** 2 )
    return 1 - (num / den)


def root_mean_square_error(sim, obs):
    obs = obs.values
    sim = sim.values  
    return np.sqrt(mean_squared_error(sim, obs))


from setuptools import setup, find_packages
import versioneer

setup(name='pyrhessys',
      version = '0.0.2',
      #version=versioneer.get_version(),
      #cmdclass=versioneer.get_cmdclass(),
      description='an Object-Oriented Python wrapper for the RHESSys model',
      url='https://github.com/DavidChoi76/pyrhessys.git',
      author='YoungDon Choi', 
      author_email='choiyd1115@gmail.com',
      license='MIT',
      packages=find_packages(),
      install_requires=[
          'numpy>=1.11.2',  
          'xarray>=0.15.0',
          'pandas',
          'netcdf4>=1.2.5',
          'numba',          
          'scipy',          
          'matplotlib',
          'hs_restclient',
          'dask',
          'distributed',
          'toolz',
          'pytest',
          'dask-jobqueue',
          'hydroeval',
          'imageio',
          'sklearn',
          'pyDOE',
          'ipyplot',
          'Bottleneck'
          ],
       extras_require={'visualization': [
          'geopandas',
          'fiona',
          'cartopy',
          'shapely',
          'seaborn',
          'geoviews',         
          ],},
      include_package_data=True,
      test_suite='pyrhessys.tests')



import os
import shutil
import pkg_resources

g2w_template = pkg_resources.resource_filename(
        __name__, 'template/g2w_template.txt')
g2w_template1 = pkg_resources.resource_filename(
        __name__, 'template/g2w_template1.txt')
g2w_template_difficult_run = pkg_resources.resource_filename(
        __name__, 'template/g2w_template_difficult_run.txt')
